[PATCH 2/8] fetch.c: Make process() look at each object only once
- From
Sergey Vlasov <vsu@altlinux.ru>
- Date
- Sep 21, 2005, 16:18 UTC
- Message-ID
- <20050921161857.96621E01C96@center4.mivlgu.local>
- In-Reply-To
- <20050921161829.GA20944@master.mivlgu.local>
The process() function is very often called multiple times for the same object (because lots of trees refer to the same blobs), but did not have a fast check for this, therefore a lot of useless calls to has_sha1_file() and parse_object() were made before discovering that nothing needs to be done.
This patch adds the SEEN flag which is used in process() to make it look at each object only once. When testing git-local-fetch on the repository of GIT, this gives a 14x improvement in CPU usage (mainly because the redundant calls to parse_object() are now avoided - parse_object() always unpacks and parses the object data, even if it was already parsed before).
---
fetch.c | 5 +++++ 1 files changed, 5 insertions(+), 0 deletions(-)
03abc6806a4433807bc21cf3f1aac2528c655c8f
diff --git a/fetch.c b/fetch.c --- a/fetch.c +++ b/fetch.c @@ -58,6 +58,7 @@ static int process_tree(struct tree *tre #define TO_FETCH 2U #define TO_SCAN 4U #define SCANNED 8U +#define SEEN 16U static struct commit_list *complete = NULL; @@ -134,6 +135,10 @@ static int process_object(struct object static int process(struct object *obj) { + if (obj->flags & SEEN) + return 0; + obj->flags |= SEEN; + if (has_sha1_file(obj->sha1)) { parse_object(obj->sha1); /* We already have it, so we should scan it now. */